-------------------------------------------
--- Author: Ketho (EU-Boulderfist)		---
--- License: Public Domain				---
--- Created: 2011.11.27					---
--- Version: 0.4 [2012.03.14]			---
-------------------------------------------
--- Curse			http://www.curse.com/addons/wow/guildleveltooltip
--- WoWInterface	http://www.wowinterface.com/downloads/info20527-GuildLevelTooltip.html
--- Request			http://www.curse.com/forums/games/world-of-warcraft/addons/148517-got-a-request

-- GuildLevelTooltip attempts to inspect "mouseover" and "target" units
-- once you have (attempted) to /inspect someone (Blizzard_InspectUI is loaded),
-- and afterwards it will only inspect on actual /inspect due to NotifyInspect's buggy nature

-- Why not to NotifyInspect() if Blizzard_InspectUI is loaded..
-- ..while InspectFrame is hidden -> InspectGuildFrame_Update() will give an error (InspectFrame.unit)
-- ..while InspectFrame is shown -> the InspectFrame will get messed up

local VERSION = 0.4
local NAME = ...

local db

local f = CreateFrame("Frame")

	--------------
	--- Events ---
	--------------

function f:ADDON_LOADED(name)
	if name == NAME then
		GuildLevelTooltipDB = GuildLevelTooltipDB or {}
		GuildLevelTooltipDB.version = VERSION
		local realm = GetRealmName()
		GuildLevelTooltipDB[realm] = GuildLevelTooltipDB[realm] or {}
		db = GuildLevelTooltipDB[realm]
	end
end

local cd = 0
local newUnit

local function Notify(unit)
	if not InspectFrame and CanInspect(unit) then
		if time() > cd then
			-- NotifyInspect() is throttled by 6 calls for every 10 sec
			cd = time() + 2
			newUnit = unit
			NotifyInspect(unit)
		end
	end
end

function f:UPDATE_MOUSEOVER_UNIT()
	Notify("mouseover")
end

function f:PLAYER_TARGET_CHANGED()
	Notify("target")
end

function f:INSPECT_READY(guid)
	-- switch to default functionality if Blizzard_InspectUI is loaded
	if InspectFrame then
		newUnit = "target" -- unit is mostly "target" on inspect
	end
	
	-- kinda annoying that INSPECT_READY passes the unit's guid instead of the unit id
	if newUnit and UnitGUID(newUnit) == guid then
		local name = GetGuildInfo(newUnit)
		-- unit is guildless; or was cleared in the meantime
		if not name then return end
		
		local _, level, xp, members = GetInspectGuildInfo(newUnit)
		-- unit is somehow out of inspect range; or inspected another than the current unit
		if level == 0 then return end
		
		db[name] = db[name] or {}
		db[name][1] = level
		db[name][2] = members
		--db[name][3] = xp -- maybe useful for future versions
		--db[name][4] = time()
		--db[name][5] = date("%Y.%m.%d %H:%M")
	end
end

function f:OnEvent(event, ...)
	self[event](self, ...)
end

f:RegisterEvent("ADDON_LOADED")
f:RegisterEvent("UPDATE_MOUSEOVER_UNIT")
f:RegisterEvent("PLAYER_TARGET_CHANGED")
f:RegisterEvent("INSPECT_READY")
f:SetScript("OnEvent", f.OnEvent)

	------------
	--- Hook ---
	------------

local function OnTooltipSetUnit()
	local unit = select(2, GameTooltip:GetUnit())
	if not unit then return end
	
	local guild = db[GetGuildInfo(unit)]
	if not guild then return end
	
	GameTooltip:AddLine(format(GUILD_LEVEL, guild[1]))
	GameTooltip:AddLine(format(INSPECT_GUILD_NUM_MEMBERS, guild[2]))
	GameTooltip:Show() -- update tooltip
end

GameTooltip:HookScript("OnTooltipSetUnit", OnTooltipSetUnit)
